
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_APPLICATION_H
#define PG_APPLICATION_H

#include "NetAddr.h"
#include "String.h"
#include "Array.h"
#include "base/CmdLine.h"
#include "FileScanner.h"
#include "runtime/BcastRcver.h"

class OLog;
class Agent;
class AgentSym;
class AgentCfg;
class SizeOpt;

// holds information required during agent configuration process
struct AgentCfg {
    AgentSym *sym;
    NetAddr *host;

    AgentCfg();
    AgentCfg(AgentSym *aSym, const NetAddr &aHost);
};


// common interface for polyclt and polysrv applications
class PolyApp: public FileScanTicker, public BcastRcver {
	public:
		PolyApp();
		virtual ~PolyApp();

		virtual void noteMsgStrEvent(BcastChannel *ch, const char *msg);
		virtual void begCycle(int readyCount);
		virtual void endCycle();
		virtual bool tick();


		int run(int argc, char *argv[]);
		void flushState();

	protected:
		bool handleCmdLine(int argc, char *argv[]);
		void makeAgents();
		void reportRUsage() const;

		virtual void configure();
		virtual void reportCfg();
		virtual void logCfg();
		virtual void step();
		
		void configureBinLog(OLog &log, const String &fname, SizeOpt &bufSz);
		void configureLogs(int prec);
		void configureHosts();
		void buildSchedule();
		void parseConfigFile(const String &fname);
		int scan(Time *toutp);
		
		void checkTiming(Time drift);
		void getIfaces();
		void getFakeIfaces();
		int countAddress(const NetAddr &addr) const;
		bool moreThanOneAddr() const;
		void addAgent(Agent *agent);
		
		virtual void startAgents();
		virtual void getOpts(Array<OptGrp*> &opts);
		virtual void wasIdle(Time tout);
		virtual void makeAgent(const AgentCfg &cfg) = 0;
		virtual void logState(OLog &log);

	protected:
		CmdLine theCmdLine;

		SchArray<in_addr> theIfaces; // list of interface or fake addresses
		Array<AgentCfg> theLocalCfgs;
		Array<Agent*> theLocals;
		String theAgentType;
		bool agentMustBind;

		Time theIdleBeg;  // the start of an idle period
		Time theIdleEnd;  // beg + idle_tout
		bool isIdle;

		int theTickCount;  // see PolyApp::tick()
		int theStateCount; // number of states [flushed]
};

#endif
