
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <iomanip.h>

#include "OLog.h"
#include "ILog.h"
#include "ObjId.h"
#include "Oid.h"

OidRec::OidRec(const ObjId &id) {
	reset();
	wid = id.world();
	sidx = -1; // XXX
	hidx = id.server();
	oid = id.name();
}

void OidRec::reset() {
	wid.clear();
	sidx = tid = oid = hidx = -1;
	repeat = cachable = hit = foreign = false;
}

bool OidRec::direct() const {
	return !embedded();
}

bool OidRec::embedded() const {
	Assert(wid);
	return wid.mutant();
}


ostream &OidRec::printOid(ostream &os) const {
	const ios::fmtflags flags = os.flags();
	const char fill = os.fill('0');
	os << hex << setfill('0') << setw(8) << oid;
	os.fill(fill);
	os.flags(flags);
	return os;
}

ostream &OidRec::printUriPath(ostream &os) const {
	os << 'w' << wid << '/';
	if (tid)
		os << 't' << tid << '/';
	printOid(os << '_');
	return os;
}

ostream &OidRec::printFlags(ostream &os) const {
	if (repeat)
		os << "rpt,";
	if (cachable)
		os << "chb,";
	if (hit)
		os << "hit,";
	if (foreign)
		os << "foreign";
	return os;
}

ostream &OidRec::print(ostream &os) const {
	os << "srv: " << sidx;
	printUriPath(os << " path: ");
	printFlags(os << " flags: ");
	return os;
}

void OidRec::store(OLog &ol) const {
	ol << sidx << tid << oid
		<< repeat << cachable << hit << foreign;
}

void OidRec::load(ILog &il) {
	il >> sidx >> tid >> oid
		>> repeat >> cachable >> hit >> foreign;
}
