
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_OID_RECORD_CLASS_H
#define PG_OID_RECORD_CLASS_H

#include "base/UniqId.h"

class ObjId;

// storage for oid properties
class OidRec {
	public:
		OidRec() { reset(); }
		OidRec(const ObjId &id);

		void reset();

		bool direct() const;
		bool embedded() const;

		ostream &print(ostream &os) const;
		ostream &printOid(ostream &os) const;
		ostream &printUriPath(ostream &os) const;
		ostream &printFlags(ostream &os) const;

		void store(OLog &ol) const;
		void load(ILog &il);

	public:
		UniqId wid;     // server world id (cached)

		int sidx;       // server rep index XXX
		int tid;        // object content type id
		int oid;        // object id

		int hidx;		// host index XXX

		bool repeat;    // was requested before
		bool cachable;  // reply is cachable
		bool hit;       // second-hand reply
		bool foreign;   // foreign request or reply
};


inline
ostream &operator <<(ostream &os, const OidRec &r) {
	return r.printOid(os);
}

inline
OLog &operator <<(OLog &ol, const OidRec &r) {
	r.store(ol);
	return ol;
}

inline
ILog &operator >>(ILog &il, OidRec &r) {
	r.load(il);
	return il;
}

#endif
