
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "StatPhaseRec.h"
#include "OLog.h"
#include "ILog.h"


StatPhaseRec::StatPhaseRec() {
}

OLog &StatPhaseRec::store(OLog &log) const {
	StatIntvlRec::store(log);
	return log 
		<< theName 
		<< theGoal
		<< theContType
		<< theConnClose
		<< theSockRdSzH << theSockWrSzH
		<< theHttpXacts
		<< theIcpXacts
		<< theRptmVsSz << theConnLifeVsUse
		;
}

ILog &StatPhaseRec::load(ILog &log) {
	StatIntvlRec::load(log);
	return log
		>> theName
		>> theGoal
		>> theContType
		>> theConnClose
		>> theSockRdSzH >> theSockWrSzH
		>> theHttpXacts
		>> theIcpXacts
		>> theRptmVsSz >> theConnLifeVsUse
		;
}

void StatPhaseRec::concat(const StatPhaseRec &r) {
	if (theName && r.theName)
		theName += "|" + r.theName;
	StatIntvlRec::concat(r);
	theGoal.concat(r.theGoal);
	join(r);
}

void StatPhaseRec::merge(const StatPhaseRec &r) {
	if (theName && r.theName)
		theName += "&" + r.theName;
	StatIntvlRec::merge(r);
	theGoal.merge(r.theGoal);
	join(r);
}

void StatPhaseRec::join(const StatPhaseRec &r) {
	if (!theName)
		theName = r.theName;

	theContType.add(r.theContType);
	theConnClose.add(r.theConnClose);
	theSockRdSzH.add(r.theSockRdSzH);
	theSockWrSzH.add(r.theSockWrSzH);
	theHttpXacts += r.theHttpXacts;
	theIcpXacts  += r.theIcpXacts;
	theRptmVsSz.add(r.theRptmVsSz);
	theConnLifeVsUse.add(r.theConnLifeVsUse);
}

ostream &StatPhaseRec::print(ostream &os, const String &pfx) const {
	os << "name:\t " << theName << endl;

	StatIntvlRec::print(os, pfx);

	theGoal.print(os, pfx + "goal.");

	theHttpXacts.print(os, "hit", "miss", pfx + "http.");
	theIcpXacts.print(os, "hit", "miss",  pfx + "icp.");

	theContType.print(os, pfx + "cont_type.");
	theConnClose.print(os, pfx + "conn_close.");
	theSockRdSzH.print(os, pfx + "so_read.size.");
	theSockWrSzH.print(os, pfx + "so_write.size.");
	theRptmVsSz.print(os, pfx + "rptm_vs_size.");
	theConnLifeVsUse.print(os, pfx + "conn_ttl_vs_use.");

	return os;
}
