
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "CacheEntry.h"
#include "Cache.h"
#include "DistrPoint.h"

Array<DistrPoint*> DistrPoint::TheWaits(1024);


DistrPoint::DistrPoint(Cache *aCache, CacheEntry *anEntry): 
	theCache(aCache), theEntry(anEntry),
	theReader(0), theWriter(0), theWaitIdx(-1) {
}

DistrPoint::~DistrPoint() {
	Assert(!theReader && !theWriter);
	if (theEntry)
		theEntry->theDistrPoint = 0;
	if (waiting())
		WillNotCall(theWaitIdx);
}

void DistrPoint::addReader(CacheReader *r) {
	Assert(theEntry && !theReader);
	theReader = r;
	if (theReadySize >= 0)
		willPush();
}

void DistrPoint::addWriter(CacheWriter *w) {
	Assert(theEntry && !theWriter);
	theWriter = w;
	// we do not know what the object size will be
	theReadySize = -1;
	theEntry->objSize(theReadySize);
}

void DistrPoint::delReader(CacheReader *r) {
	Assert(r && r == theReader);
	r->theDistrPoint = 0;
	theReader = 0;
	if (!theWriter)
		stop();
}

void DistrPoint::delWriter(CacheWriter *w) {
	Assert(w && w == theWriter);
	w->theDistrPoint = 0;
	theWriter = 0;
	if (!theReader)
		stop();
	// XXX: else should queue theReader->noteWriterLeft()
}

void DistrPoint::noteDataReady(Size readySize) {
	Assert(theReadySize <= readySize);
	theReadySize = readySize;
	willPush();
}

void DistrPoint::pushData() {
	if (theReader)
		theReader->noteCacheReady();
	// it is unsafe to do anything here
}

void DistrPoint::stop() {
	Assert(theEntry);
	Assert(!theReader && !theWriter);
	if (waiting())
		WillNotCall(theWaitIdx);
	theEntry->stopDistributing(this);
	// stopDistributing destroys us
}

void DistrPoint::willPush() {
	if (!waiting())
		theWaitIdx = WillCall(this);
}

void DistrPoint::callWaiting() {
	Assert(waiting());
	theWaitIdx = -1;
	pushData();
}

int DistrPoint::WillCall(DistrPoint *dp) {
	TheWaits.append(dp);
	return TheWaits.count();
}

void DistrPoint::WillNotCall(int &widx) {
	Assert(widx >= 0);
	if (widx < TheWaits.count())
		TheWaits[widx] = 0;
	widx = -1;
}

void DistrPoint::CallAll() {
	// It is probably safe to call all waits,
	// including those added during the loop.
	// Be fair, use SCAN approach, not "shortest seek"
	while (int count = TheWaits.count()) {
		for (int i = 0; i < count; ++i) {
			if (DistrPoint *dp = TheWaits[count])
				dp->callWaiting();
		}
		TheWaits.shift(count);
	}
}
