
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_DISTRIBUTION_POINT_H
#define PG_DISTRIBUTION_POINT_H

#include "Array.h"

class Cache;
class CacheEntry;
class CacheReader;
class CacheWriter;

// rendezvous point for object readers (serving side) and
// object writers (requesting side)

class DistrPoint {
	public:
		static void CallAll(); // activates all waiting actions

	public:
		DistrPoint(Cache *aCache, CacheEntry *anEntry);
		~DistrPoint();

		Cache *cache() { return theCache; }
		CacheEntry *entry() { return theEntry; }
		Size readySize() const { return theReadySize; }
		bool reader() const { return theReader; }
		bool writer() const { return theWriter; }

		void addReader(CacheReader *r);
		void addWriter(CacheWriter *w);
		void delReader(CacheReader *r);
		void delWriter(CacheWriter *w);

		// called by the writer (or reader on a hit)
		void noteDataReady(Size readySize);

	protected:
		static int WillCall(DistrPoint *dp);
		static void WillNotCall(int &widx);

	protected:
		bool waiting() const { return theWaitIdx >= 0; }
		void callWaiting();

		void willPush();
		void pushData();
		void stop();

	protected:
		static Array<DistrPoint*> TheWaits; // postponed "recursive" calls

	protected:
		Cache *theCache;
		CacheEntry *theEntry;
		CacheReader *theReader; // XXX: only one reader is supported!
		CacheWriter *theWriter;

		Size theReadySize;      // object [content] size
		int theWaitIdx;         // reservation with TheWaits
};

#endif
