
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "AsyncClt.h"
#include "SyncClt.h"
#include "base/OLog.h"
#include "runtime/LogComment.h"
#include "runtime/SharedOpts.h"
#include "runtime/StatIntvl.h"
#include "ServerRep.h"
#include "IcpCltXact.h"
#include "CltOpts.h"
#include "CltXact.h"
#include "CltConnMgr.h"
#include "PolyClt.h"
#include "WorkSetFreezer.h"

#include "PglSemx.h"
#include "RobotSym.h"

#include "polyLogCats.h"
#include "gadgets.h"
#include "rndDistrs.h"
#include "runtime/globals.h"
#include "runtime/shutdown.h"

typedef XactFarmT<CltXact, CltXact> CltXactFarm;


PolyClt::PolyClt() {
	theAgentType = "Robot";
}

void PolyClt::makeAgent(const AgentCfg &cfg) {
	Assert(cfg.sym);
	
	const RobotSym *rcfg = &(const RobotSym&)cfg.sym->cast("Robot");

	Client *client = 0;
	if (RndDistr *iad = rcfg->reqInterArrival())
		client = new AsyncClt(iad);
	else
		client = new SyncClt();

	Assert(client);
	client->configure(rcfg, *cfg.host, agentMustBind);

	addAgent(client);
}

void PolyClt::configure() {

	PolyApp::configure();

	// working set freezer
	if (PglSemx::TheWorkSetLen >= 0)
		(void)new WorkSetFreezer(PglSemx::TheWorkSetLen);

	// new oid prefetching
	ServerRep::ConfigureNids(TheCltOpts.theNidsPrefDepth, TheOpts.theNidsPerMsgMax);

	Client::Farm(new CltXactFarm);
	IcpCltXact::TheTimeout = TheCltOpts.theIcpTout;
	StatIntvl::ActiveCat(lgcCltSide);
	StatIntvl::TheReportCat = lgcCltSide;
}

void PolyClt::getOpts(Array<OptGrp*> &opts) {
	PolyApp::getOpts(opts);
	opts.append(&TheCltOpts);
}

void PolyClt::reportCfg() {
	PolyApp::reportCfg();
}

void PolyClt::logState(OLog &log) {
	PolyApp::logState(log);
	Client::LogState(log);
}

int main(int argc, char *argv[]) {
	PolyClt app;
	return app.run(argc, argv);
}
