
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include <math.h>

#include "xstd/Assert.h"
#include "xstd/Rnd.h"
#include "xstd/gadgets.h"
#include "pgl/PopDistr.h"
#include "pgl/PopModelSym.h"
#include "client/PopModel.h"


PopModel::PopModel(): theDistr(0), theHotSetProb(-1), theHotSetFrac(1) {
}

void PopModel::configure(const PopModelSym *cfg) {
	theDistr = cfg->popDistr();
	cfg->hotSetFrac(theHotSetFrac);
	cfg->hotSetProb(theHotSetProb);

	if (!theDistr) {
		cerr << cfg->loc() << "popularity distribution is required if PopModel is used" << endl;
		exit(-2);
	}
}

int PopModel::choose(RndGen &rng, int lastOid, int wss, int hotSetPos) {
	Assert(wss != 0);

	// adjust params if we need a "hot" object
	if (theHotSetProb > 0 && rng.event(theHotSetProb)) {
		wss = wss > 0 ? (int)rint(wss*theHotSetFrac) : (lastOid+1);
		lastOid = Max(wss, hotSetPos);
	}

	const int offset = (0 < wss && wss < lastOid) ? lastOid-wss : 0;
	const int oname = offset + theDistr->choose(rng, lastOid - offset);

	Assert(0 < oname && oname <= lastOid);
	return oname;
}

