
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_SERVER_REPRESENTATIVE_H
#define PG_SERVER_REPRESENTATIVE_H

// client side "representative" of an origin server 
// holds client side knowledge about a server

#include "Ring.h"
#include "NetAddr.h"
#include "base/UniqId.h"

class OLog;
class RepHdr;


class ServerRep {
	public:
		static int TheSharedWss;  // sum of all server wss's
		static void ConfigureNids(int prefetchLmt, int perReqMax);

	public:
		ServerRep(const NetAddr &anAddr, int hostIdx);

		const NetAddr &addr() const { return theAddr; }
		const UniqId &id() const { return theId; }
		int hostIdx() const { return theHostIdx; }

		bool haveNewOids() const { return theNewOids.count(); }
		bool haveFailedOids() const { return theFailedOids.count(); }
		int wss() const { return theWss; }
		int accesses() const { return theAccCount; }

		int oidShortage(); // changes number of "pending" oid reqs

		int newOid();
		int failedOid();
		int secOid() const { return theSecOid; }
		int maxNewOidCap() const { return theNewOids.capacity(); }
		int pendCount() const { return thePendCnt; }
		int hotSetPos() const { return theHotSetPos; }

		void noteNewOids(int startOid, int askCnt, int gotCnt);
		void noteFailedOid(int oid);
		void syncSrvInfo(const RepHdr &rep);
		void noteNoNewOids(); // client ran out of new oids
		void noteAccess() { theAccCount++; }

		void freezeWss();      // based on local info only

		void store(OLog &log) const;
		void load(ILog &log);

	protected:
		void syncSrvId(const UniqId &aSrvId);
		void syncWss(int wss);

	protected:
		static int TheNewOidsPrefetchDepth; // control new oid prefetching
		static int TheNewOidsPerReqMax;     //  algorithm

	protected:
		NetAddr theAddr;
		UniqId theId;
		Ring<int> theNewOids;
		Ring<int> theFailedOids;
		int theHostIdx;    // index into the HostMap
		int theLastNewOid; // last new oid returned by newOid()
		int theSecOid;     // our idea of server's "secure" oid
		int theAccCount;   // accesses to this server
		int theWss;        // number of oids in server's working set
		int theHotSetPos;  // as set by the server

	private:
		int thePendCnt; // outstanding reqs for new oids
};

#endif
