
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglIntSym.h"
#include "PglRec.h"
#include "PglRecSym.h"
#include "PglStringSym.h"
#include "PglArraySym.h"
#include "PglNetAddrSym.h"


NetAddrSym::NetAddrSym(): RecSym("addr", new PglRec) {
	theRec->bAdd("string", "host", 0);
	theRec->bAdd("int", "port", 0);
}

NetAddrSym::NetAddrSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool NetAddrSym::isA(const String &type) const {
	return RecSym::isA(type) ||
		type == "NetAddr" || type == "addr";
}

SynSym *NetAddrSym::clone(const String &type) const {
	if (isA(type))
		return new NetAddrSym(this->type(), theRec->clone());
	return ExpressionSym::clone(type);
}

NetAddr NetAddrSym::val() const {
	Assert(theRec);
	SynSymTblItem *host = 0;
	SynSymTblItem *port = 0;
	Assert(theRec->find("host", host) && theRec->find("port", port));
	return NetAddr(
		host->sym() ? 
			((const StringSym&)host->sym()->cast("String")).val() : String(0),
		port->sym() ?
			((const IntSym&)port->sym()->cast("Int")).val() : -1);
}

void NetAddrSym::val(const NetAddr &addr) {
	Assert(theRec);
	SynSymTblItem *host = 0;
	SynSymTblItem *port = 0;
	Assert(theRec->find("host", host) && theRec->find("port", port));
	delete host->sym();
	delete port->sym();
	host->sym(addr.addrA() ? new StringSym(addr.addrA()) : 0);
	port->sym(addr.port() >= 0 ? new IntSym(addr.port()) : 0);
}

ExpressionSym *NetAddrSym::bnOper(const String &op, const SynSym &s) const {
	NetAddrSym *na2s = (NetAddrSym*)s.clone("addr");
	if (op != ".." || !na2s) {
		delete na2s;
		return ExpressionSym::bnOper(op, s);
	}

	const NetAddr na1 = val();
	if (na1.addrN().s_addr == INADDR_NONE)
		cerr << loc() << "applying range operator to an address requires IP address at the start of the range" << endl << xexit;

	const NetAddr na2 = na2s->val();
	if (na2.addrN().s_addr == INADDR_NONE)
		cerr << loc() << "applying range operator to an address requires IP address at the end of the range" << endl << xexit;

	const unsigned long net = na1.net();
	if (net != na2.net())
		cerr << loc() << "both ends of an IP range must have the same network number" << endl << xexit;

	if (na1.port() != na2.port())
		cerr << loc() << "both ends of an IP range must have the same port number" << endl << xexit;

	const unsigned long lna1 = na1.lna();
	const unsigned long lna2 = na2.lna();

	// note: the code will generate .255 addresses if ends cross
	// class B networks; is this desirable? should we allow for
	// such crossings?
	ArraySym *arr = new ArraySym("addr");
	NetAddrSym h;
	unsigned long lna = lna1;
	for (int i = 0; i < 255; ++i) { // limit to 255 addresses
		h.val(NetAddr::MakeAddr(net, lna, na1.port()));
		arr->add(h);

		if (lna < lna2)
			lna++;
		else
		if (lna > lna2)
			lna--;
		else
			break;
	}

	Assert(arr->count()); // empty ranges are probably impossible
	delete na2s;
	return arr;
}

ostream &NetAddrSym::print(ostream &os, const String &) const {
	return os << val();
}
