
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include "pgl/PglRec.h"
#include "pgl/PglStringSym.h"
#include "pgl/PglArraySym.h"
#include "pgl/GoalSym.h"
#include "pgl/StatsSampleSym.h"
#include "pgl/PhaseSym.h"


PhaseSym::PhaseSym(): RecSym("Phase", new PglRec) {
	theRec->bAdd("string", "name", 0);
	theRec->bAdd("Goal", "goal", new GoalSym);

	theRec->bAdd("float", "recur_factor", 0);
	theRec->bAdd("float", "load_factor_beg", 0);
	theRec->bAdd("float", "load_factor_end", 0);

	theRec->bAdd("StatsSample[]", "stats_samples", 0);
	theRec->bAdd("bool", "log_stats", 0);
}

PhaseSym::PhaseSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool PhaseSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Phase";
}

SynSym *PhaseSym::clone(const String &type) const {
	if (isA(type))
		return new PhaseSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

void PhaseSym::name(const String &aName) {
	SynSymTblItem *ni = 0;
	Assert(theRec->find("name", ni));
	delete ni->sym();
	ni->sym(new StringSym(aName));
}

String PhaseSym::name() const {
	return getString("name");
}

GoalSym *PhaseSym::goal() const {
	SynSymTblItem *gi = 0;
	Assert(theRec->find("goal", gi));
	Assert(gi->sym());
	return &(GoalSym&)gi->sym()->cast("Goal");
}

bool PhaseSym::recurFactor(double &rf) const {
	return getDouble("recur_factor", rf);
}

bool PhaseSym::loadFactorBeg(double &lf) const {
	return getDouble("load_factor_beg", lf);
}

bool PhaseSym::loadFactorEnd(double &lf) const {
	return getDouble("load_factor_end", lf);
}

bool PhaseSym::logStats(bool &ls) const {
	return getBool("log_stats", ls);
}

bool PhaseSym::statsSamples(Array<const StatsSampleSym*> &samples) const {
	SynSymTblItem *ssi = 0;
	Assert(theRec->find("stats_samples", ssi));
	if (!ssi->sym())
		return false; // undefined

	// build address array and selector
	ArraySym &ssa = (ArraySym&)ssi->sym()->cast("Array");
	samples.stretch(ssa.count());
	for (int i = 0; i < ssa.count(); ++i) {
		const StatsSampleSym * ss =
			&((const StatsSampleSym&)ssa[i]->cast("StatsSample"));
		samples.append(ss);
	}

	return samples.count();
}
