
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "runtime/Connection.h"

#include "base/ObjId.h"
#include "Client.h"
#include "DistrPoint.h"
#include "PxyCltXact.h"

void PxyCltXact::reset() {
	CltXact::reset();
	CacheWriter::reset();
}

Cache *PxyCltXact::cache() {
	return theOwner->cache(); 
}

// also called from noteCacheReady
void PxyCltXact::consume(Size size) {
	Assert(theDistrPoint);
	CltXact::consume(size);
	if (theConsumedSize) {
		//cerr << here << theDistrPoint << ": ready: " << theConsumedSize - theRepHdr.theHdrSize << endl;
		theDistrPoint->noteDataReady(
			Max(Size(0), theConsumedSize - theRepHdr.theHdrSize));
	}
}

void PxyCltXact::noteReaderLeft() {
	// here is the place to implement quick_abort
}

void PxyCltXact::newState(State aState) {
	CltXact::newState(aState);
	if (theState == stHdrWaiting) {
		Assert(!theDistrPoint);
		if (CacheEntry *e = cache()->cached(theOidRec))
			cache()->purgeEntry(e); // we are going to update it
		theDistrPoint = cache()->addWriter(theOidRec, this);
		Assert(theDistrPoint);
	} else
	if (theState == stDone) {
		if (theDistrPoint) {
			CacheEntry *e = theDistrPoint->entry();
			Assert(e->objSize() < 0);

			// XXX: also need to purge on error
			// (need a single point where errors are delivered)
			if (theRepSize < 0) {
				if (e->cached())
					cache()->purgeEntry(e);
			} else {
				const Size sz = theRepSize - theRepHdr.theHdrSize;
				Assert(sz >= 0);
				e->objSize(sz);
				if (!e->cached() && theOidRec.cachable)
					cache()->cacheEntry(e);
			}

			theDistrPoint->delWriter(this);
		}
	}
}
