
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "Rnd.h"
#include "String.h"
#include "Socket.h"
#include "LogComment.h"
#include "base/RndPermut.h"
#include "Agent.h"
#include "AgentSym.h"
#include "SocketSym.h"

#include "globals.h"

int Agent::TheLastSeqvId = 0;

Agent::Agent(): theThinkDistr(0), theCache(0) {
	theSeqvId = ++TheLastSeqvId;
}

Agent::~Agent() {
}

void Agent::configure(const AgentSym *cfg, const NetAddr &aHost, SockOpt &opt) {
	Assert(TheFileScanner);
	Assert(cfg);

	theHost = aHost;
	theKind = cfg->kind();

	if (UniqId id = cfg->world())
		theId = id;
	else
		theId.create();

	if (!theThinkDistr) {
		theThinkDistr = cfg->xactThink();
		if (theThinkDistr)
			theThinkDistr->rndGen(GenRndGen("agent_xact_think"));
	}

	// socket options
	bool ng = true;
	if (cfg->socket()->nagle(ng))
		opt.nagle = ng ? 1 : -1;
	cfg->socket()->lingerTout(opt.linger);
}

void Agent::cache(Cache *aCache) {
	Assert(!theCache);
	theCache = aCache;
}

void Agent::start() {
	Comment(2) << "starting agent[" << seqvId() 
		<< " / " << theId
		<< "] `" << theKind << "'"
		<< " on " << theHost << endc;
}
