
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "base/ErrorStrTbl.h"
#include "LogComment.h"
#include "ErrorMgr.h"
#include "polyBcastChannels.h"

ErrorMgr TheErrorMgr;


ErrorMgr::ErrorMgr(): theVerbLvl(1), theStrs(0), theTotCnt(0) {
}

void ErrorMgr::configure(const ErrorStrTbl *aStrs) {
	Assert(aStrs);
	Assert(!theStrs);
	theStrs = aStrs;

	theCnts.maxErrNo(aStrs->maxErrNo());
	theExplDlines.stretch(theCnts.count());
	theExplDlines.count(theExplDlines.capacity());
}

void ErrorMgr::canonError(int &e) const {
	if (e < 0 || e > theCnts.maxErrNo())
		e = theCnts.maxErrNo();
}

// note: side effect of increasing reporting deadline
bool ErrorMgr::needExpl(int e) {
	canonError(e);
	if (theVerbLvl > 0 && theCnts[e] >= theExplDlines[e]) {
		theExplDlines[e] += 1 + theExplDlines[e]/theVerbLvl;
		return true;
	}
	return false;
}

void ErrorMgr::noteError(int e) {
	canonError(e);
	theCnts.record(e);
	theTotCnt++;
	Broadcast(TheErrChannel, e);
}

void ErrorMgr::reportError(const SrcLoc &loc, int e) {
	canonError(e);
	Comment(0)
		<< loc << "error: "
		<< theCnts[e] << '/' << theTotCnt
		<< " (" << e << ") "
		<< (*theStrs)[e]
		<< endc;
}

const ErrorStrTbl *ErrorMgr::strings() const {
	return theStrs;
}

const ErrorStat &ErrorMgr::stats() const {
	return theCnts;
}


/* global routines */

bool ReportErrorLoc(const SrcLoc &loc, int e) {
	TheErrorMgr.noteError(e);
	if (TheErrorMgr.needExpl(e)) {
		TheErrorMgr.reportError(loc, e);
		return true;
	}
	return false;
}

void FatalErrorLoc(const SrcLoc &loc, int e) {
	TheErrorMgr.noteError(e);
	TheErrorMgr.reportError(loc, e);
	Comment(0) << "last error (" << e << ") was fatal; terminating..." << endc;
	exit(e);
}
