
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include <iomanip.h>

#include "Clock.h"
#include "Connection.h"
#include "Xaction.h"
#include "IcpXaction.h"
#include "base/StatIntvlRec.h"
#include "StatIntvl.h"

#include "globals.h"
#include "gadgets.h"
#include "polyBcastChannels.h"
#include "polyLogCats.h"


int StatIntvl::TheReportCat = lgcAll;
Array<bool> StatIntvl::IsActiveCat(lgcEnd);


void StatIntvl::ActiveCat(int cat) {
	Assert(0 <= cat && cat < lgcEnd);
	IsActiveCat.put(true, cat);
}

StatIntvl::StatIntvl() {
	Assert(TheConnOpenChannel);
	theChannels
		<< TheConnOpenChannel << TheConnEstChannel << TheConnCloseChannel
		<< TheXactBegChannel << TheXactEndChannel << TheXactErrChannel
		<< TheIcpXactBegChannel << TheIcpXactEndChannel << TheIcpXactErrChannel
		<< TheWaitBegChannel << TheWaitEndChannel
		;
}

void StatIntvl::setDuration(Time start) {
	const Time dur = TheClock.time() >= start ? TheClock - start : Time(0,0);
	for (int i = 0; i < IsActiveCat.count(); ++i) {
		if (IsActiveCat[i])
			getRec(i).theDuration = dur;
	}
}

void StatIntvl::storeAll(OLog &ol, int tag) const {
	for (int i = 0; i < IsActiveCat.count(); ++i) {
		if (IsActiveCat[i]) {
			ol << bege(tag, i);
			getRec(i).store(ol);
			ol << ende;
		}
	}
}

void StatIntvl::noteConnEvent(BcastChannel *ch, const Connection *conn) {
	Assert(conn);
	StatIntvlRec &rec = getRec(conn->logCat());

	if (ch == TheConnOpenChannel)
		++rec.theOpenLvl;
	else 
	if (ch == TheConnEstChannel)
		++rec.theEstbLvl;
	else {
		Assert(ch == TheConnCloseChannel);
		if (!conn->bad()) {
			rec.theConnLifeTm.record((TheClock - conn->openTime()).msec());
			rec.theConnUseCnt.record(conn->useCnt());
		}
		if (conn->ioCnt())
			--rec.theEstbLvl;
		--rec.theOpenLvl;
	}
}

void StatIntvl::noteXactEvent(BcastChannel *ch, const Xaction *x) {
	Assert(x);
	StatIntvlRec &rec = getRec(x->logCat());

	if (ch == TheXactBegChannel) {
		++rec.theXactLvl;
	} else
	if (ch == TheXactEndChannel) {
		const OidRec &oid = x->oidRec();
		// XXX: calculate and use "ideal" time here
		rec.theIdealHR.record(Time(0,0), x->repSize(), oid.repeat && oid.cachable);
		rec.theRealHR.record(x->lifeTime(), x->repSize(), oid.hit);
		rec.theChbR.record(x->lifeTime(), x->repSize(), oid.cachable);
		if (!oid.hit && oid.cachable)
			rec.theFill.record(x->lifeTime(), x->repSize());

		rec.theXactCnt++;
		--rec.theXactLvl;
	} else
	if (ch == TheWaitBegChannel) {
		++rec.theWaitLvl;
	} else
	if (ch == TheWaitEndChannel) {
		--rec.theWaitLvl;
	} else {
		Assert(ch == TheXactErrChannel);
		rec.theXactErrCnt++;
		if (x && x->started())
			--rec.theXactLvl;
	}
}

void StatIntvl::noteIcpXactEvent(BcastChannel *ch, const IcpXaction *x) {
	Assert(x);
	StatIntvlRec &rec = getRec(x->logCat());

	if (ch == TheIcpXactEndChannel) {
		if (x->timedout())
			rec.theIcpStat.recordTimeout();
		else
			rec.theIcpStat.record(x->lifeTime(), x->repSize(), x->hit());
	}
}
