
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_STAT_PHASE_H
#define PG_STAT_PHASE_H

#include "xstd/Array.h"
#include "base/ErrorStat.h"
#include "base/StatPhaseRec.h"
#include "runtime/Goal.h"
#include "runtime/StatIntvl.h"

class StatPhaseMgr;
class StatsSampleCfg;
class PhaseSym;
class StatsSampleSym;

// stats phase is a named [long] interval during which
// stats are accumulated; the phase goal may not be [just] time based
// the stats are logged when instructed by the manager
// when phase goal is reached stat phase terminates notifying the manager

class StatPhase: public StatIntvl {
	public:
		StatPhase();
		virtual ~StatPhase();

		void configure(const PhaseSym *ps, const StatPhase *prevPh);
		const char *name() const { return theName; }
		bool used() const { return wasUsed; }
		double loadFactorBeg() const;
		double loadFactorEnd() const;

		Time duration() const;
		int xactCnt() const;
		BigSize fillSz() const;
		int errCnt() const;
		double errRatio() const;

		void statsLogged(bool are);
		void name(const String &aName);

		bool varyLoad() const { return theLoadFactK; }
		double loadFactor(Time lastFactTm, int lastFactCnt) const;
		double recurFactor() const;

		void lock();
		void unlock();

		void start(StatPhaseMgr *aMgr, const StatPhase *prevPhase);
		void flush();

		virtual void wakeUp(const Alarm &alarm);

		virtual void noteConnEvent(BcastChannel *ch, const Connection *c);
		virtual void noteXactEvent(BcastChannel *ch, const Xaction *x);
		virtual void noteIcpXactEvent(BcastChannel *ch, const IcpXaction *x);
		virtual void noteErrNoEvent(BcastChannel *ch, int errNo);

		// should these be replaced with BcastChannels
		void noteSockRead(Size sz, int cat) { theRecs[cat]->theSockRdSzH.record(sz); }
		void noteSockWrite(Size sz, int cat) { theRecs[cat]->theSockWrSzH.record(sz); }

		void reportCfg(ostream &os) const;

	protected:
		void configureSamples(const PhaseSym *cfg);

		virtual StatIntvlRec &getRec(int cat);
		virtual const StatIntvlRec &getRec(int cat) const;

		bool locked() const;
		bool checkpoint();
		void stop();
		void report() const;

	protected:
		Array<StatPhaseRec*> theRecs;
		Array<StatsSampleCfg*> theSamples;

		StatPhaseMgr *theMgr;
		ErrorStat theErrs;     // not a part of PhaseRec, logged separately
		String theName;
		Goal theGoal;          // configurable GoalRec with reached()

		bool wasUsed;          // was listening for events
		bool logStats;         // log stats when flushed
		bool wasStopped;       // stop() was called

		double theLoadFactBeg;
		double theLoadFactEnd;
		double theLoadFactK;

		double theRecurFact;

		int theLockCount;
};


#endif
