
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "xstd.h"

#include "Assert.h"
#include "AlarmClock.h"

AlarmClock TheAlarmClock;

/* Alarm */

void Alarm::ring() {
	Assert(theSleeper);
	theSleeper->wakeUp(*this);
	clear();
}

void Alarm::snooze() {
	Assert(theSleeper);
	theSleeper->snooze(*this);
	clear();
}

void Alarm::set(Time aTime, AlarmUser *aSleeper) {
	Assert(aSleeper);

	theSleeper = aSleeper;
	theTime = aTime;
}


/* AlarmUser */

AlarmUser::AlarmUser(): thePendAlarmCnt(0) {
}

AlarmUser::~AlarmUser() {
	cancelAlarms();
}

void AlarmUser::wakeUp(const Alarm &) {
	thePendAlarmCnt--;
	Assert(thePendAlarmCnt >= 0);
}

// ignore this alarm
void AlarmUser::snooze(const Alarm &) {
	thePendAlarmCnt--;
	Assert(thePendAlarmCnt >= 0);
}

void AlarmUser::sleepTill(Time time) {
	TheAlarmClock.setAlarm(Alarm(time, this));
	thePendAlarmCnt++;
}

void AlarmUser::sleepFor(Time delay) {
	sleepTill(TheClock + delay);
}

void AlarmUser::cancelAlarms() {
	if (thePendAlarmCnt)
		TheAlarmClock.cancelAll(this);
	Assert(!thePendAlarmCnt);
}


/* AlarmClock */

AlarmClock::AlarmClock() {
}

void AlarmClock::setAlarm(const Alarm &alarm) { 
	if (alarm) {
		theQueue.add(alarm);
		if (theRingTime < 0 || alarm.time() < theRingTime)
			noteNewRingTime(alarm.time());
	}
}

void AlarmClock::cancelAll(const AlarmUser *sleeper) {
	for (int i = theQueue.count()-1; i >= 0 && sleeper->alarmWaiting(); --i)
		if (theQueue.at(i).sleeper() == sleeper)
			theQueue.at(i).snooze();
}

// returns time left (negative if late)
void AlarmClock::ring() {
	while (!theQueue.empty()) {
		if (Alarm &alarm = theQueue.top()) {
			noteNewRingTime(alarm.time());
			if (theTimeLeft > 0) // not ready yet
				break;
			theRingTime = Time();
			alarm.ring();
		}
		theQueue.skip();
	}
}

void AlarmClock::update(Time curTime) {
	Clock::update(curTime);
	if (theRingTime >= 0) {
		theTimeLeft = theRingTime - theCurTime;
		if (theRingTime <= theCurTime)
			ring();
	}
}

void AlarmClock::noteNewRingTime(Time newTime) {
	static Time lastRingTime;
	Assert(newTime >= lastRingTime);

	theTimeLeft = newTime - theCurTime;
	theRingTime = newTime;
}
