
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "xstd.h"

#include <string.h>

#include "BitMask.h"
#include "Assert.h"


Size BitMask::size() const {
	return BinCount(theCapacity) * sizeof(*theBins);
}

void BitMask::resize(int aCapacity) {
	if (aCapacity == theCapacity)
		return;

	delete[] theBins;
	if (aCapacity > 0) {
		theBins = new Bin[BinCount(aCapacity)];
		theCapacity = aCapacity;
		clear();
	} else {
		theBins = 0;
		theCapacity = 0;
	}
}

void BitMask::grow(int aCapacity, int offset) {
	Assert(aCapacity > theCapacity);

	BitMask old;
	old.takeOver(*this);
	this->resize(aCapacity);

	const int y = offset + old.capacity();
	for (int x = offset; x < y; ++x) {
		if (old.isSet(x % old.capacity()))
			this->beSet(x % capacity());
	}
}

void BitMask::clear() {
	if (theBins)
		memset(theBins, 0, size());
}

void BitMask::takeOver(BitMask &from) {
	Assert(!theBins);
	theBins = from.theBins;
	theCapacity = from.theCapacity;

	from.theBins = 0;
	from.theCapacity = 0;
}
