
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef FUTURE_CLASS_H
#define FUTURE_CLASS_H

#include "Array.h"

// "Future plan": a circular buffer; new entries are inserted
// at random positions relative to "now"; "now" or "next step"
// is at position "zero"
// for simplicity, assume that at most MAX_INT items are inserted
// over the life time of an object
template <class Item>
class Future {
	public:
		Future(int aCapacity = 0): theLog(aCapacity), theOffset(0) {}

		void capacity(int aCapacity) { Assert(!theOffset); theLog.stretch(aCapacity); }
		int capacity() const { return theLog.capacity(); }

		const Item &operator [](int off) const { return theLog[(theOffset + off) % theLog.capacity()]; }
		Item &operator [](int off) { return theLog[(theOffset + off) % theLog.capacity()]; }
		void shift(Item i) { theLog[theOffset++ % theLog.capacity()] = i; }

	protected:
		Array<Item> theLog;
		int theOffset;      // "now" or the start of a plan
};

#endif
